@@ -2,7 +2,7 @@
 """
 Main script with the class definition for accessing Zernike polynomial initialization, calculation and plotting.
 
-@author: Sergei Klykov, @year: 2022 \n
+@author: Sergei Klykov, @year: 2023 \n
 @licence: MIT \n
 
 """
@@ -16,10 +16,12 @@ import matplotlib.pyplot as plt
 
 # %% Local (package-scoped) imports
 if __name__ == "__main__" or __name__ == Path(__file__).stem or __name__ == "__mp_main__":
-    from calculations.calc_zernike_pol import normalization_factor, radial_polynomial, triangular_function
+    from calculations.calc_zernike_pol import (normalization_factor, radial_polynomial, triangular_function,
+                                               triangular_derivative, radial_derivative)
     from plotting.plot_zerns import plot_sum_fig, subplot_sum_on_fig
 else:
-    from .calculations.calc_zernike_pol import normalization_factor, radial_polynomial, triangular_function
+    from .calculations.calc_zernike_pol import (normalization_factor, radial_polynomial, triangular_function,
+                                                triangular_derivative, radial_derivative)
     from .plotting.plot_zerns import plot_sum_fig, subplot_sum_on_fig
 
 # %% Module parameters
@@ -97,108 +99,129 @@ class ZernPol:
         ZernPol class' instance.
 
         """
-        __orders_specified = False; __index_specified = False; key = ""
+        key = ""
         # Zernike polynomial specified with key arguments m, n - check firstly for these parameters
-        if "n" in kwargs.keys() or "radial_order" in kwargs.keys():
-            # get the actual name of the key for radial order
-            if "n" in kwargs.keys():
-                key = "n"
-            else:
-                key = "radial_order"
-            if isinstance(kwargs.get(key), int):
-                self.__n = kwargs.get(key)  # radial order acknowledged
-                # Below each time key arguments are checked in the list of keys
-                if ("m" in kwargs.keys() or "l" in kwargs.keys() or "azimuthal_order" in kwargs.keys()
-                   or "angular_frequency" in kwargs.keys()):
-                    if "m" in kwargs.keys():
-                        key = "m"
-                    elif "l" in kwargs.keys():
-                        key = "l"
-                    elif "azimuthal_order" in kwargs.keys():
-                        key = "azimuthal_order"
+        if len(kwargs.keys()) == 2:
+            if "n" in kwargs.keys() or "radial_order" in kwargs.keys():
+                if self.__initialized:
+                    raise ValueError("The polynomial has been already initialized, but radial_order/n/m parsed")
+                else:
+                    # get the actual name of the key for radial order
+                    if "n" in kwargs.keys():
+                        key = "n"
                     else:
-                        key = "angular_frequency"
+                        key = "radial_order"
                     if isinstance(kwargs.get(key), int):
-                        self.__m = kwargs.get(key)  # azimuthal order acknowledged
-                        # Checking that the provided orders are reasonable
-                        if not((self.__n - abs(self.__m)) % 2 == 0):  # see [1]
-                            raise ValueError("Failed sanity check: n - |m| == even number")
-                        elif self.__n < 0:
-                            raise ValueError("Failed sanity check: order n less than 0")
-                        elif self.__n == 0 and self.__m != 0:
-                            raise ValueError("Failed sanity check: when n == 0, m should be also == 0")
-                        # m and n specified correctly, calculate other properties - various indices
+                        self.__n = kwargs.get(key)  # radial order acknowledged
+                        # Below each time key arguments are checked in the list of keys
+                        if ("m" in kwargs.keys() or "l" in kwargs.keys() or "azimuthal_order" in kwargs.keys()
+                           or "angular_frequency" in kwargs.keys()):
+                            if "m" in kwargs.keys():
+                                key = "m"
+                            elif "l" in kwargs.keys():
+                                key = "l"
+                            elif "azimuthal_order" in kwargs.keys():
+                                key = "azimuthal_order"
+                            else:
+                                key = "angular_frequency"
+                            if isinstance(kwargs.get(key), int):
+                                self.__m = kwargs.get(key)  # azimuthal order acknowledged
+                                # Checking that the provided orders are reasonable
+                                if not (self.__n - abs(self.__m)) % 2 == 0:  # see [1]
+                                    raise ValueError("Failed sanity check: n - |m| == even number")
+                                elif self.__n < 0:
+                                    raise ValueError("Failed sanity check: order n less than 0")
+                                elif self.__n == 0 and self.__m != 0:
+                                    raise ValueError("Failed sanity check: when n == 0, m should be also == 0")
+                                # m and n specified correctly, calculate other properties - various indices
+                                else:
+                                    self.__initialized = True  # set the flag to True, polynomial class initialized
+                                    # Calculation of various indices according to [1]
+                                    self.__osa_index = ZernPol.get_osa_index(self.__m, self.__n)
+                                    self.__noll_index = ZernPol.get_noll_index(self.__m, self.__n)
+                                    self.__fringe_index = ZernPol.get_fringe_index(self.__m, self.__n)
+                            else:
+                                raise ValueError("Azimuthal order m provided not as an integer")
                         else:
-                            __orders_specified = True  # set the flag to True
-                            # Calculation of various indices according to [1]
-                            self.__osa_index = ZernPol.get_osa_index(self.__m, self.__n)
+                            # the n order defined, but m hasn't been found
+                            self.__n = 0
+                    else:
+                        raise ValueError("Radial order n provided not as an integer")
+        elif len(kwargs.keys()) == 1:
+            # OSA / ANSI index used for Zernike polynomial initialization
+            if ("osa_index" in kwargs.keys() or "osa" in kwargs.keys() or "ansi_index" in kwargs.keys()
+               or "ansi" in kwargs.keys()):
+                if self.__initialized:
+                    raise ValueError("The polynomial has been already initialized, but osa_index/osa... parsed")
+                else:
+                    if "osa_index" in kwargs.keys():
+                        key = "osa_index"
+                    elif "osa" in kwargs.keys():
+                        key = "osa"
+                    elif "ansi_index" in kwargs.keys():
+                        key = "ansi_index"
+                    elif "ansi" in kwargs.keys():
+                        key = "ansi"
+                    if isinstance(kwargs.get(key), int):
+                        osa_i = kwargs.get(key)
+                        if osa_i < 0:
+                            raise ValueError("OSA / ANSI index should be non-negative integer")
+                        else:
+                            self.__osa_index = osa_i; self.__initialized = True
+                            self.__m, self.__n = ZernPol.index2orders(osa_index=self.__osa_index)
                             self.__noll_index = ZernPol.get_noll_index(self.__m, self.__n)
                             self.__fringe_index = ZernPol.get_fringe_index(self.__m, self.__n)
                     else:
-                        raise ValueError("Azimuthal order m provided not as an integer")
+                        raise ValueError("OSA / ANSI index provided not as an integer")
+            # Noll index used for Zernike polynomial initialization
+            elif "noll_index" in kwargs.keys() or "noll" in kwargs.keys():
+                if self.__initialized:
+                    raise ValueError("The polynomial has been already initialized, but noll_index/noll parsed")
                 else:
-                    # the n order defined, but m hasn't been found
-                    self.__n = 0
-            else:
-                raise ValueError("Radial order n provided not as an integer")
-        # OSA / ANSI index used for Zernike polynomial initialization
-        elif ("osa_index" in kwargs.keys() or "osa" in kwargs.keys() or "ansi_index" in kwargs.keys()
-              or "ansi" in kwargs.keys()):
-            if __orders_specified:
-                raise ValueError("The polynomial has been already initialized with (m, n) parameters")
-            else:
-                if "osa_index" in kwargs.keys():
-                    key = "osa_index"
-                elif "osa" in kwargs.keys():
-                    key = "osa"
-                elif "ansi_index" in kwargs.keys():
-                    key = "ansi_index"
-                elif "ansi" in kwargs.keys():
-                    key = "ansi"
-                if isinstance(kwargs.get(key), int):
-                    self.__osa_index = kwargs.get(key); __index_specified = True
-                    self.__m, self.__n = ZernPol.index2orders(osa_index=self.__osa_index)
-                    self.__noll_index = ZernPol.get_noll_index(self.__m, self.__n)
-                    self.__fringe_index = ZernPol.get_fringe_index(self.__m, self.__n)
+                    if "noll_index" in kwargs.keys():
+                        key = "noll_index"
+                    elif "noll" in kwargs.keys():
+                        key = "noll"
+                    if isinstance(kwargs.get(key), int):
+                        noll_i = kwargs.get(key)
+                        if noll_i < 1:
+                            raise ValueError("Noll index should be not less than 1 integer")
+                        else:
+                            self.__noll_index = noll_i; self.__initialized = True
+                            self.__m, self.__n = ZernPol.index2orders(noll_index=self.__noll_index)
+                            self.__osa_index = ZernPol.get_osa_index(self.__m, self.__n)
+                            self.__fringe_index = ZernPol.get_fringe_index(self.__m, self.__n)
+                    else:
+                        raise ValueError("Noll index provided not as an integer")
+            # Fringe / Univ. of Arizona index used for Zernike polynomial initialization
+            elif "fringe_index" in kwargs.keys() or "fringe" in kwargs.keys():
+                if self.__initialized:
+                    raise ValueError("The polynomial has been already initialized, but fringe_index/fringe parsed")
                 else:
-                    raise ValueError("OSA / ANSI index provided not as an integer")
-        # Noll index used for Zernike polynomial initialization
-        elif "noll_index" in kwargs.keys() or "noll" in kwargs.keys():
-            if __orders_specified:
-                raise ValueError("The polynomial has been already initialized with (m, n) parameters")
-            else:
-                if "noll_index" in kwargs.keys():
-                    key = "noll_index"
-                elif "noll" in kwargs.keys():
-                    key = "noll"
-                if isinstance(kwargs.get(key), int):
-                    self.__noll_index = kwargs.get(key); __index_specified = True
-                    self.__m, self.__n = ZernPol.index2orders(noll_index=self.__noll_index)
-                    self.__osa_index = ZernPol.get_osa_index(self.__m, self.__n)
-                    self.__fringe_index = ZernPol.get_fringe_index(self.__m, self.__n)
-                else:
-                    raise ValueError("Noll index provided not as an integer")
-        # Fringe / Univ. of Arizona index used for Zernike polynomial initialization
-        elif "fringe_index" in kwargs.keys() or "fringe" in kwargs.keys():
-            if __orders_specified:
-                raise ValueError("The polynomial has been already initialized with (m, n) parameters")
-            else:
-                if "fringe_index" in kwargs.keys():
-                    key = "fringe_index"
-                elif "fringe" in kwargs.keys():
-                    key = "fringe"
-                if isinstance(kwargs.get(key), int):
-                    self.__fringe_index = kwargs.get(key); __index_specified = True
-                    self.__m, self.__n = ZernPol.index2orders(fringe_index=self.__fringe_index)
-                    self.__osa_index = ZernPol.get_osa_index(self.__m, self.__n)
-                    self.__noll_index = ZernPol.get_noll_index(self.__m, self.__n)
-                else:
-                    raise ValueError("Fringe index provided not as an integer")
+                    if "fringe_index" in kwargs.keys():
+                        key = "fringe_index"
+                    elif "fringe" in kwargs.keys():
+                        key = "fringe"
+                    if isinstance(kwargs.get(key), int):
+                        fringe_i = kwargs.get(key)
+                        if fringe_i < 1:
+                            raise ValueError("Fringe index should be not less than 1 integer")
+                        else:
+                            self.__fringe_index = fringe_i; self.__initialized = True
+                            self.__m, self.__n = ZernPol.index2orders(fringe_index=self.__fringe_index)
+                            self.__osa_index = ZernPol.get_osa_index(self.__m, self.__n)
+                            self.__noll_index = ZernPol.get_noll_index(self.__m, self.__n)
+                    else:
+                        raise ValueError("Fringe index provided not as an integer")
         else:
-            raise ValueError("Key arguments haven't been parsed / recognized, see docs for acceptable values")
+            raise ValueError("Length of provided key arguments are not equal 1 (OSA/Fringe/Noll index) or 2 (m, n orders)")
         # Also raise the ValueError if the ZernPol hasn't been initialized by orders / indices
-        if not __index_specified and not __orders_specified:
-            raise ValueError("The initialization parameters for Zernike polynomial hasn't been parsed")
+        if not self.__initialized:
+            raise ValueError("The initialization parameters for Zernike polynomial hasn't been parsed / recognized")
+        else:
+            if self.__n >= 26 and not self.__n == abs(self.__m) and not self.__n - 2 == abs(self.__m):
+                message = f"Initialized polynomial with order {self.__n} possibly will take long time to compute a value"
+                warnings.warn(message)
 
     def get_indices(self):
         """
@@ -259,8 +282,8 @@ class ZernPol:
         """
         Calculate Zernike polynomial value(-s) within the unit circle.
 
-        Calculation up to 8th order of Zernike polynomials performed by exact equations, after - iteratively, using
-        the equations borrowed from the Reference [1] below.
+        Calculation up to 10th order of Zernike function performed by exact equations from Ref.[2],
+        after - using the recurrence equations taken from the Ref.[1].
 
         References
         ----------
@@ -272,17 +295,17 @@ class ZernPol:
         Parameters
         ----------
         r : float or numpy.ndarray
-            Radius from the unit circle, float or array of values on which the Zernike polynomial is calculated.
+            Radius (radii) from unit circle or the range [0.0, 1.0], float / array, for which the polynomial is calculated.
         theta : float or numpy.ndarray
-            Theta - angle in radians, float or array of angles on which the Zernike polynomial is calculated.
+            Theta - angle in radians from the range [0, 2*pi], float or array, for which the polynomial is calculated.
             Note that the theta counting is counterclockwise, as it is default for the matplotlib library.
 
         Raises
         ------
         ValueError
-            Check the Error signature for reasons, most probably some data inconsistency detected.
+            Check the Error stack trace for reasons of raising, most probably input parameters aren't acceptable.
         Warning
-            If the theta angles cover the range more than 2*pi (period).
+            If the theta angles lie outside the range [0, 2*pi] (entire period).
 
         Returns
         -------
@@ -291,33 +314,156 @@ class ZernPol:
 
         """
         # Checking input parameters for avoiding errors and unexpectable values
-        # Trying to convert known data types into numpy arrays
-        if not isinstance(r, np.ndarray) and not isinstance(r, float):
-            if isinstance(r, list) or isinstance(r, tuple):
-                r = np.asarray(r)  # convert list and tuple to np.array
-            else:
-                r = float(r)  # attempt to convert r to float number, will raise ValueError if it's impossible
-        if not isinstance(theta, np.ndarray) and not isinstance(theta, float):
-            if isinstance(theta, list) or isinstance(theta, tuple):
-                theta = np.asarray(theta)  # convert list and tuple to np.array
-            else:
-                theta = float(theta)  # attempt to convert to float number, will raise ValueError if it's impossible
-        # Checking the basic demands on acceptable data types - np.ndarray and float
-        elif isinstance(r, np.ndarray) and isinstance(theta, np.ndarray):
+        # Check radii type and that they are not lying outside range [0.0, 1.0] - unit circle
+        r = ZernPol._check_radii(r)
+        # Checking that angles lie in the range [0, 2*pi] and their type
+        theta = ZernPol._check_angles(theta)
+        # Checking coincidence of shapes if theta and r are arrays
+        if isinstance(r, np.ndarray) and isinstance(theta, np.ndarray):
             if r.shape != theta.shape:
                 raise ValueError("Shape of input arrays r and theta is not equal")
-        elif isinstance(r, np.ndarray):
-            if np.min(r) < 0.0 or np.max(r) > 1.0:
-                raise ValueError("Minimal or maximal value of radiuses laying outside unit circle [0.0, 1.0]")
-        elif isinstance(r, float):
-            if 0.0 > r > 1.0:
-                raise ValueError("Minimal or maximal value of radiuses laying outside unit circle [0.0, 1.0]")
-        elif isinstance(theta, np.ndarray):
-            if np.max(theta) + abs(np.min(theta)) > 2*np.pi:
-                warnings.warn("Theta angles defined in range more than 2*pi, check them")
         # Calculation using imported function
         return normalization_factor(self)*radial_polynomial(self, r)*triangular_function(self, theta)
 
+    def radial(self, r):
+        """
+        Calculate R(m, n) - radial Zernike function value(-s) within the unit circle.
+
+        Calculation up to 10th order of Zernike function performed by exact equations from Ref.[2],
+        after - using the recurrence equations taken from the Ref.[1].
+
+        References
+        ----------
+        [1] Shakibaei B.H., Paramesran R. "Recursive formula to compute Zernike radial polynomials" (2013) \n
+        [2] Lakshminarayanan V., Fleck A. "Zernike polynomials: a guide" (2011) \n
+        [3] Andersen T. B. "Efficient and robust recurrence relations for the Zernike
+        circle polynomials and their derivatives in Cartesian coordinates" (2018) \n
+
+        Parameters
+        ----------
+        r : float or numpy.ndarray
+            Radius (radii) from unit circle or the range [0.0, 1.0], float / array, for which the function is calculated.
+
+        Raises
+        ------
+        ValueError
+            The probable reasons: radius (radii) doesn't belong to a unit circle, input type isn't acceptable.
+
+        Returns
+        -------
+        float or numpy.ndarray
+            Calculated Zernike radial function value(-s) on provided float values / arrays of radiuses.
+
+        """
+        # Check radii type and that they are not lying outside range [0.0, 1.0] - unit circle
+        r = ZernPol._check_radii(r)
+        # Calculation using imported function
+        return radial_polynomial(self, r)
+
+    def triangular(self, theta):
+        """
+        Calculate triangular Zernike function value(-s) within the unit circle.
+
+        Parameters
+        ----------
+        theta : float or numpy.ndarray
+            Theta - angle in radians from the range [0, 2*pi], float or array, for which the polynomial is calculated.
+            Note that the theta counting is counterclockwise, as it is default for the matplotlib library.
+
+        Raises
+        ------
+        ValueError
+            Most probably, raised if the conversion to float number is failed. \n
+            It happens when input parameter is not float, numpy.ndarray, list or tuple.
+        Warning
+            If the theta angles lie outside the range [0, 2*pi] (entire period).
+
+        Returns
+        -------
+        float or numpy.ndarray
+            Calculated value(-s) of Zernike triangular function on provided angle.
+
+        """
+        # Check theta type and that angles are lying in the single period range [0, 2pi]
+        theta = ZernPol._check_angles(theta)
+        # Calculation using imported function
+        return triangular_function(self, theta)
+
+    def radial_dr(self, r):
+        """
+        Calculate derivative of radial Zernike polynomial value(-s) within the unit circle.
+
+        Calculation up to 10th order of Zernike polynomials performed by exact equations,
+        after - using the recurrence equations.
+
+        References
+        ----------
+        Same as for the method "radial" or "polynomial value"
+
+        Parameters
+        ----------
+        r : float or numpy.ndarray
+            Radius (radii) from unit circle or the range [0.0, 1.0], float / array, for which the polynomial is calculated.
+
+        Raises
+        ------
+        ValueError
+            The probable reasons: radius (radii) doesn't belong to a unit circle, input type isn't acceptable.
+
+        Returns
+        -------
+        float or numpy.ndarray
+            Calculated derivative of Zernike radial function value(-s) on provided float values / arrays of radiuses.
+
+        """
+        # Checking input parameters for avoiding errors and unexpectable values
+        r = ZernPol._check_radii(r)
+        return radial_derivative(self, r)
+
+    def triangular_dtheta(self, theta):
+        """
+        Calculate derivative from triangular function on angle theta.
+
+        Parameters
+        ----------
+        theta : float or numpy.ndarray
+            Theta - angle in radians from the range [0, 2*pi], float or array, for which the function is calculated.
+            Note that the theta counting is counterclockwise, as it is default for the matplotlib library.
+
+        Raises
+        ------
+        ValueError
+            Most probably, raised if the conversion to float number is failed. \n
+            It happens when input parameter is not float, numpy.ndarray, list or tuple.
+        Warning
+            If the theta angles lie outside the range [0, 2*pi] (entire period).
+
+        Returns
+        -------
+        float or numpy.ndarray
+            Calculated derivative value(-s) of Zernike triangular function on provided angle.
+
+        """
+        # Check input parameter type and attempt to convert to acceptable types
+        theta = ZernPol._check_angles(theta)
+        return triangular_derivative(self, theta)
+
+    def normf(self):
+        """
+        Calculate normalization factor for the Zernike polynomial calculated according to the Reference below.
+
+        References
+        ----------
+        [1] Shakibaei B.H., Paramesran R. "Recursive formula to compute Zernike radial polynomials" (2013)
+
+        Returns
+        -------
+        float
+            Normalization factor calculated according to the Reference.
+
+        """
+        return normalization_factor(self)
+
     # %% Static methods
     @staticmethod
     def get_osa_index(m: int, n: int) -> int:
@@ -582,7 +728,7 @@ class ZernPol:
         Returns
         -------
         Sum of Zernike polynomials
-            Depending on the input values and parameter get_surface - can be float, 1D or 2D numpy.ndarrays.
+            Depending on the input values and parameter get_surface - can be: float, 1D or 2D numpy.ndarrays.
 
         """
         S = 0.0  # default value - sum
@@ -758,6 +904,110 @@ class ZernPol:
                                         zernikes_sum_surface.Theta, show_range_colorbar=show_range)
         return figure
 
+    @staticmethod
+    def _plot_zernikes_half_pyramid():
+        """
+        Generate halb-pyramid with Zernikes polynomials.
+
+        Returns
+        -------
+        None.
+
+        """
+        n_cols = 7; n_rows = 6; fig = plt.figure(figsize=(10, 7.8))
+        axes = fig.subplots(n_rows, n_cols, subplot_kw=dict(projection='polar'), squeeze=False)
+        zps = []; ampls = [1.0]; k = 1  # k for OSA indexing
+        ignored_column = n_cols - 2
+        for i in range(len(axes)):
+            for j in range(len(axes[0])):
+                axes[i, j].grid(False)  # demanded by pcolormesh function, if not called - deprecation warning
+                if j > ignored_column-1:
+                    zps = [ZernPol(osa=k)]
+                    zernike_surface, r, theta = ZernPol.gen_zernikes_surface(coefficients=ampls, polynomials=zps)
+                    axes[i, j].pcolormesh(theta, r, zernike_surface, cmap=plt.cm.coolwarm, shading='nearest')
+                    k += 1
+                axes[i, j].axis('off')  # off polar coordinate axes
+            ignored_column -= 1
+        fig.subplots_adjust(left=0, bottom=0, right=1, top=1)
+        fig.tight_layout()
+
+    @staticmethod
+    def _check_radii(radii):
+        """
+        Perform check of type of input, attempt to convert to numpy.ndarray or float, check that r is inside [0.0, 1.0].
+
+        Parameters
+        ----------
+        radii : float or numpy.ndarray
+            Radii from the unit circle (range [0.0, 1.0]), float or array, for which the Zernike polynomials are defined.
+
+        Raises
+        ------
+        ValueError
+            The probable reasons: radius (radii) doesn't belong to a unit circle, input type isn't acceptable.
+
+        Returns
+        -------
+        radii : float or numpy.ndarray
+            Return radii, converted, if it's applicable, to float or numpy.ndarray.
+
+        """
+        # Trying to convert known (list, tuple) data types into numpy, if they provided as input
+        if not isinstance(radii, np.ndarray) and not isinstance(radii, float):
+            if isinstance(radii, list) or isinstance(radii, tuple):
+                radii = np.asarray(radii)  # convert list or tuple to np.array
+            else:
+                radii = float(radii)  # attempt to convert r to float number, will raise ValueError if it's impossible
+        # Checking that radii or radius lie in the range [0.0, 1.0]
+        if isinstance(radii, np.ndarray):
+            if np.min(radii) < 0.0 or np.max(radii) > 1.0:
+                raise ValueError("Minimal or maximal value of radii laying outside unit circle [0.0, 1.0]")
+        elif isinstance(radii, float):
+            if 0.0 > radii > 1.0:
+                raise ValueError("Radius laying outside unit circle [0.0, 1.0]")
+        return radii
+
+    @staticmethod
+    def _check_angles(angles):
+        """
+        Perform check of type of input, attempt to convert to numpy.ndarray or float, check that angles is inside [0, 2pi].
+
+        Parameters
+        ----------
+        angles : float or numpy.ndarray
+            Theta polar coordinates from unit circle there Zernike polynomials are defined.
+
+        Raises
+        ------
+        ValueError
+            Most probably, raised if the conversion to float number is failed. \n
+            It happens when input parameter is not float, numpy.ndarray, list or tuple.
+        Warning
+            If the theta angles lie outside the range [0, 2*pi] (entire period).
+
+        Returns
+        -------
+        angles : float or numpy.ndarray
+            Return angles, converted, if it's applicable, to float or numpy.ndarray.
+
+        """
+        # Check input parameter type and attempt to convert to acceptable types
+        if not isinstance(angles, np.ndarray) and not isinstance(angles, float):
+            if isinstance(angles, list) or isinstance(angles, tuple):
+                angles = np.asarray(angles)  # convert list or tuple to np.array
+            else:
+                angles = float(angles)  # attempt to convert to float number, will raise ValueError if it's impossible
+        # Checking that angles lie in the range [0, 2*pi]
+        if isinstance(angles, np.ndarray):
+            if np.max(angles) + abs(np.min(angles)) > 2.0*np.pi:
+                warnings.warn("Theta angles defined in range outside of interval [0, 2*pi]")
+            elif np.max(angles) > 2.0*np.pi or np.min(angles) < 0.0:
+                warnings.warn("Max or min of theta angles lies outside of interval [0, 2*pi]")
+        elif isinstance(angles, float):
+            if angles < 0.0 or angles > 2.0*np.pi:
+                warnings.warn("Max or min of theta angles lies outside of interval [0, 2*pi]")
+        return angles
+
 
 # %% Test functions for the external call
 def check_conformity():
@@ -834,14 +1084,18 @@ def check_conformity():
 # %% Tests
 if __name__ == "__main__":
     check_conformity()  # testing initialization
-    # Check plotting functions
-    plt.close("all")
-    zp = ZernPol(m=0, n=4); ZernPol.plot_zernike_polynomial(zp)  # basic plotting
+    plt.close("all")  # Check plotting functions
+    # zp = ZernPol(m=0, n=4); ZernPol.plot_zernike_polynomial(zp)  # basic plotting
     # Two plots should look similar below, using different methods to call
     zp1 = ZernPol(m=0, n=2); zp2 = ZernPol(m=-3, n=3); coefficients = [1.0, 1.0]; polynomials = [zp1, zp2]
-    fig = plt.figure(figsize=(4, 4)); fig = ZernPol.plot_sum_zernikes_on_fig(coefficients, polynomials, fig)
-    fig1 = plt.figure(figsize=(4, 4)); zern_surf = ZernPol.gen_zernikes_surface(coefficients, polynomials)
-    fig1 = ZernPol.plot_sum_zernikes_on_fig(coefficients, polynomials, fig1, show_range=False)
-    # fig2 = plt.figure(figsize=(2.8, 2.8)); zp3 = ZernPol(m=2, n=2); polynomials = [zp3]; coefficients = [1.0]
-    # fig2 = ZernPol.plot_sum_zernikes_on_fig(coefficients, polynomials, fig2, show_range=False)
-    plt.show()
+    # fig = plt.figure(figsize=(4, 4)); fig = ZernPol.plot_sum_zernikes_on_fig(coefficients, polynomials, fig)
+    # fig1 = plt.figure(figsize=(4, 4)); zern_surf = ZernPol.gen_zernikes_surface(coefficients, polynomials)
+    # fig1 = ZernPol.plot_sum_zernikes_on_fig(coefficients, polynomials, fig1, show_range=False)
+    # ZernPol._plot_zernikes_half_pyramid()
+    fig3 = plt.figure(figsize=(2, 2)); zp3 = ZernPol(osa=9); polynomials = [zp3]; coefficients = [1.0]
+    fig3 = ZernPol.plot_sum_zernikes_on_fig(coefficients, polynomials, fig3, show_range=False)
+    fig3.subplots_adjust(0, 0, 1, 1); plt.show()
+    # Check that warning is raised
+    z = ZernPol(n=26, m=-10); print(z.radial(0.85))
+    # Check that warning not raised
+    ZernPol(n=32, m=32); ZernPol(n=28, m=-26)
